#!/usr/bin/perl -w

use Test::More 'no_plan';

package Catch;

sub TIEHANDLE {
    my($class, $var) = @_;
    return bless { var => $var }, $class;
}

sub PRINT  {
    my($self) = shift;
    ${'main::'.$self->{var}} .= join '', @_;
}

sub READ {}
sub READLINE {}
sub GETC {}

my $Original_File = 'root/usr/lib/perl5/site_perl/esmith/FormMagick/Panel/useraccounts.pm';

package main;

# pre-5.8.0's warns aren't caught by a tied STDERR.
$SIG{__WARN__} = sub { $main::_STDERR_ .= join '', @_; };
tie *STDOUT, 'Catch', '_STDOUT_' or die $!;
tie *STDERR, 'Catch', '_STDERR_' or die $!;

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 95 root/usr/lib/perl5/site_perl/esmith/FormMagick/Panel/useraccounts.pm

$ENV{ESMITH_ACCOUNT_DB} = "10e-smith-base/accounts.conf";
$ENV{ESMITH_CONFIG_DB} = "10e-smith-base/configuration.conf";

open DATA, "echo '<form></form>'|";
use_ok('esmith::FormMagick::Panel::useraccounts');
use vars qw($panel);
ok($panel = esmith::FormMagick::Panel::useraccounts->new(), 
    "Create panel object");
close DATA;
isa_ok($panel, 'esmith::FormMagick::Panel::useraccounts');
$panel->{cgi} = CGI->new();
$panel->parse_xml();

{ package esmith::FormMagick::Panel::useraccounts;
  our $accountdb;
  ::isa_ok($accountdb, 'esmith::AccountsDB');
}


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 134 root/usr/lib/perl5/site_perl/esmith/FormMagick/Panel/useraccounts.pm
$panel->print_user_table;
like($_STDOUT_, qr/bart/, "Found usernames in user table output");
like($_STDOUT_, qr/ff0000/, "Found red 'reset password' output");

    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 225 root/usr/lib/perl5/site_perl/esmith/FormMagick/Panel/useraccounts.pm

my $fm = esmith::FormMagick::Panel::useraccounts->new();
$fm->{cgi} = CGI->new("");
print_acctName_field($fm);
like($_STDOUT_, qr/text.*acctName/, "print text field if acctName not set");
like($_STDOUT_, qr/create/, "action=create if acctName not set");
$fm->{cgi}->param(-name => 'acctName', -value => 'foo');
$fm->{cgi}->param(-name => 'action', -value => 'modify');
print_acctName_field($fm);
like($_STDOUT_, qr/hidden.*acctName/, "print hidden field if acctName is set");
like($_STDOUT_, qr/modify/, "action=modify if acctName already set");


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 316 root/usr/lib/perl5/site_perl/esmith/FormMagick/Panel/useraccounts.pm

my $fm = esmith::FormMagick::Panel::useraccounts->new();
$fm->{cgi} = CGI->new("");
$fm->print_groupMemberships_field();
like($_STDOUT_, qr/simpsons/, "Found simpsons in group list");
like($_STDOUT_, qr/flanders/, "Found flanders in group list");
$fm->{cgi}->param(-name => 'acctName', -value => 'rod');
$fm->print_groupMemberships_field();
like($_STDOUT_, qr/checked value="flanders"/, "Checked flanders group for user rod");


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 382 root/usr/lib/perl5/site_perl/esmith/FormMagick/Panel/useraccounts.pm

my $fm = esmith::FormMagick::Panel::useraccounts->new();
$fm->{cgi} = CGI->new({ acctName => 'bart' });
print_page_description($fm, "reset");
like($_STDOUT_, qr/bart/, "print_page_description prints username");
like($_STDOUT_, qr/Bart Simpson/, "print_page_description prints name");
like($_STDOUT_, qr/RESET_DESC/, "print_page_description prints description");


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 435 root/usr/lib/perl5/site_perl/esmith/FormMagick/Panel/useraccounts.pm
my $fm = esmith::FormMagick::Panel::useraccounts->new();
$fm->{cgi} = CGI->new("");
is(get_ldap_value($fm, "Dept"), "Main", "Pick up default value from LDAP");
$fm->{cgi} = CGI->new({ acctName => 'bart' });
is(get_ldap_value($fm, "Dept"), undef, "Don't pick up LDAP data if username provided");

    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 480 root/usr/lib/perl5/site_perl/esmith/FormMagick/Panel/useraccounts.pm

my $fm = esmith::FormMagick::Panel::useraccounts->new();

$fm->{cgi} = CGI->new({ 
    acctName => 'skud', 
    FirstName => 'Kirrily',
    LastName => 'Robert' 
});

is  (pseudonym_clash($fm, 'Kirrily'), "OK", "New name doesn't clash pseudonyms");

$fm->{cgi} = CGI->new({ 
    acctName => 'bart2', 
    FirstName => 'Bart',
    LastName => 'Simpson' 
});

isnt(pseudonym_clash($fm, 'Bart'), "OK", "Existing pseudonym with non-matching username causes clash");

$fm->{cgi} = CGI->new({ 
    acctName => 'bart', 
    FirstName => 'Bart',
    LastName => 'Simpson' 
});

is(pseudonym_clash($fm, 'Bart'), "OK", "Existing pseudonym with matching username shouldn't clash");


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 560 root/usr/lib/perl5/site_perl/esmith/FormMagick/Panel/useraccounts.pm
is  (emailforward($panel, ""), "OK", "blank email address is OK");
is  (emailforward($panel, 'foo@bar.com'), "OK", 'foo@bar.com email address is OK');
is  (emailforward($panel, "bart"), "OK", "user bart address email is OK");
is  (emailforward($panel, "simpsons"), "OK", "group simpsons email address is OK");
isnt(emailforward($panel, "simp sons"), "OK", "group 'simp sons' address is an invalid email address");
is  (emailforward($panel, "bart.simpson"), "OK", "pseudonym bart.simpson email address is OK");
is  (emailforward($panel, "bart_simpson"), "OK", "pseudonym bart_simpson email address is OK");
isnt(emailforward($panel, "foo!bar"), "OK", "foo!bar is an invalid email address");

    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 993 root/usr/lib/perl5/site_perl/esmith/FormMagick/Panel/useraccounts.pm
is($panel->validate_acctName('foo'), 'OK', 'validate_acctName');
isnt($panel->validate_acctName('3amigos'), 'OK', ' .. cannot start with number');
isnt($panel->validate_acctName('betty ford'), 'OK', ' .. cannot contain space');

    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 1018 root/usr/lib/perl5/site_perl/esmith/FormMagick/Panel/useraccounts.pm

ok(($panel->validate_acctName_length('foo') eq 'OK'), "a short account name passes");
ok(($panel->validate_acctName_length('fooooooooooooooooo') eq 'ACCOUNT_TOO_LONG'), "a long account name fails");


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

